@@ -1,4 +1,4 @@
-#!/usr/bin/python
+#!/usr/bin/python3
 # -*- coding: utf-8 -*-
 #
 # «recovery_advanced_gtk» - Dell Recovery Media Builder
@@ -25,20 +25,19 @@
 
 import dbus
 import os
-import sys
-import gtk
+from gi.repository import Gtk
 import subprocess
 
 from Dell.recovery_gtk import DellRecoveryToolGTK, translate_widgets
 from Dell.recovery_basic_gtk import BasicGeneratorGTK
 
-from Dell.recovery_common import (UIDIR, GIT_TREES, UP_FILENAMES,
-                                  increment_bto_version)
+from Dell.recovery_common import (UIDIR, UP_FILENAMES,
+                                  dbus_sync_call_signal_wrapper)
 
 try:
     from aptdaemon import client
-    from aptdaemon.gtkwidgets import AptProgressDialog
-except ImportError:
+    from aptdaemon.gtk3widgets import AptProgressDialog
+except:
     pass
 
 #Translation support
@@ -52,21 +51,13 @@ class AdvancedGeneratorGTK(BasicGeneratorGTK):
     def __init__(self, utility, recovery, version, media, target,
                  overwrite, xrev, branch):
         """Inserts builder widgets into the Gtk.Assistant"""
-        try:
-            import vte
-        except ImportError:
-            header = _("python-vte is missing")
-            body = _("Builder mode requires python-vte to function")
-            self.show_alert(gtk.MESSAGE_ERROR, header, body,
-                parent=None)
-            sys.exit(1)
 
         #Run the normal init first
         BasicGeneratorGTK.__init__(self, utility, recovery,
                                    version, media, target, overwrite)
 
         #Build our extra GUI in
-        self.builder_widgets = gtk.Builder()
+        self.builder_widgets = Gtk.Builder()
         self.builder_widgets.add_from_file(os.path.join(UIDIR, 'builder.ui'))
         self.builder_widgets.connect_signals(self)
 
@@ -82,32 +73,32 @@ OEM FID framework & driver package set into a customized \
 OS media image.  You will have the option to \
 create an USB key or DVD image."))
 
-        self.file_dialog = gtk.FileChooserDialog("Choose Item",
+        self.file_dialog = Gtk.FileChooserDialog("Choose Item",
                                         None,
-                                        gtk.FILE_CHOOSER_ACTION_OPEN,
-                                        (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
-                                        gtk.STOCK_OPEN, gtk.RESPONSE_OK))
-        self.file_dialog.set_default_response(gtk.RESPONSE_OK)
-
-        #setup the VTE window for GIT stuff
-        self.vte = vte.Terminal()
-        self.builder_widgets.get_object('fetch_expander').add(self.vte)
-        self.vte.show()
-        self.vte.connect("child-exited", self.fid_vte_handler)
+                                        Gtk.FileChooserAction.OPEN,
+                                        (Gtk.STOCK_CANCEL, Gtk.ResponseType.CANCEL,
+                                        Gtk.STOCK_OPEN, Gtk.ResponseType.OK))
+        self.file_dialog.set_default_response(Gtk.ResponseType.OK)
 
         #setup transient windows
         for window in ['builder_add_dell_recovery_window',
-                       'srv_dialog',
-                       'builder_vte_window']:
+                       'srv_dialog']:
             self.builder_widgets.get_object(window).set_transient_for(wizard)
 
         #insert builder pages in reverse order
+        titles = {'application_page' : _("Application Packages"),
+                 'driver_page' : _("Driver Packages"),
+                 'up_page' : _("Utility Partition"),
+                 'fid_page' : _("FID Content"),
+                 'base_page' : _("Base OS Image")
+                }
         for page in ['application_page',
                      'driver_page',
                      'up_page',
                      'fid_page',
                      'base_page']:
             wizard.insert_page(self.builder_widgets.get_object(page), 0)
+            wizard.set_page_title(wizard.get_nth_page(0), titles[page])
 
         #improve the summary
         self.widgets.get_object('version_hbox').show()
@@ -115,13 +106,11 @@ create an USB key or DVD image."))
         #builder variable defaults
         self.xrev = xrev
         self.branch = branch
-        self.builder_fid_overlay = ''
         self.builder_base_image = ''
         self.bto_base = False
         self.bto_up = ''
         self.add_dell_recovery_deb = ''
         self.apt_client = None
-        self.git_pid = -1
 
         self.builder_widgets.connect_signals(self)
 
@@ -143,26 +132,17 @@ create an USB key or DVD image."))
         if page == self.builder_widgets.get_object('base_page'):
             if self.rp:
                 self.builder_widgets.get_object('recovery_hbox').set_sensitive(True)
-            file_filter = gtk.FileFilter()
+            file_filter = Gtk.FileFilter()
             file_filter.add_pattern("*.iso")
             self.file_dialog.set_filter(file_filter)
-            wizard.set_page_title(page, _("Choose Base OS Image"))
 
         elif page == self.builder_widgets.get_object('fid_page'):
-            wizard.set_page_title(page, _("Choose FID Overlay"))
-            self.builder_widgets.get_object('install_git_button').hide()
-            self.builder_widgets.get_object('add_dell_recovery_button').hide()
-
-            for operating_system in GIT_TREES:
-                if operating_system == self.distributor:
-                    self.builder_widgets.get_object('git_url').set_text(GIT_TREES[operating_system])
             self.fid_toggled(None)
 
         elif page == self.builder_widgets.get_object('up_page'):
-            wizard.set_page_title(page, _("Choose Utility Partition"))
             if self.up:
                 self.builder_widgets.get_object('utility_hbox').set_sensitive(True)
-            file_filter = gtk.FileFilter()
+            file_filter = Gtk.FileFilter()
             for item in UP_FILENAMES:
                 pattern = item.split('.')[1]
                 file_filter.add_pattern("*.%s" % pattern)
@@ -171,9 +151,8 @@ create an USB key or DVD image."))
             self.up_toggled(None)
 
         elif page == self.builder_widgets.get_object('driver_page'):
-            wizard.set_page_title(page, _("Choose Driver Packages"))
-            self.file_dialog.set_action(gtk.FILE_CHOOSER_ACTION_OPEN)
-            filefilter = gtk.FileFilter()
+            self.file_dialog.set_action(Gtk.FileChooserAction.OPEN)
+            filefilter = Gtk.FileFilter()
             filefilter.add_pattern("*.tgz")
             filefilter.add_pattern("*.tar.gz")
             filefilter.add_pattern("*.deb")
@@ -185,9 +164,8 @@ create an USB key or DVD image."))
             wizard.set_page_complete(page, True)
 
         elif page == self.builder_widgets.get_object('application_page'):
-            wizard.set_page_title(page, _("Choose Application Packages"))
-            self.file_dialog.set_action(gtk.FILE_CHOOSER_ACTION_OPEN)
-            filefilter = gtk.FileFilter()
+            self.file_dialog.set_action(Gtk.FileChooserAction.OPEN)
+            filefilter = Gtk.FileFilter()
             filefilter.add_pattern("*.tgz")
             filefilter.add_pattern("*.tar.gz")
             filefilter.add_pattern("*.zip")
@@ -210,9 +188,6 @@ create an USB key or DVD image."))
             else:
                 output_text += "<b>" + _("Base Image") + "</b>: "
                 output_text += self.builder_base_image + '\n'
-            if self.builder_fid_overlay:
-                output_text += "<b>" + _("FID Overlay") + "</b>: "
-                output_text += self.builder_fid_overlay + '\n'
 
             if self.bto_up:
                 output_text +="<b>" + _("Utility Partition: ") + '</b>'
@@ -260,10 +235,9 @@ create an USB key or DVD image."))
             srv = model.get_value(iterator, 1)
             application_fish_list[path] = srv
             iterator = model.iter_next(iterator)
-            
+
         function = 'assemble_image'
         args = (self.builder_base_image,
-                self.builder_fid_overlay,
                 driver_fish_list,
                 application_fish_list,
                 self.add_dell_recovery_deb,
@@ -276,7 +250,7 @@ create an USB key or DVD image."))
         """Browses all files under a particular filter"""
         response = self.file_dialog.run()
         self.file_dialog.hide()
-        if response == gtk.RESPONSE_OK:
+        if response == Gtk.ResponseType.OK:
             return self.file_dialog.get_filename()
         else:
             return None
@@ -290,7 +264,7 @@ create an USB key or DVD image."))
 
         if self.builder_widgets.get_object('up_files_radio').get_active():
             self.builder_widgets.get_object('up_details_label').set_markup("")
-            self.file_dialog.set_action(gtk.FILE_CHOOSER_ACTION_OPEN)
+            self.file_dialog.set_action(Gtk.FileChooserAction.OPEN)
             up_browse_button.set_sensitive(True)
             wizard.set_page_complete(up_page, False)
         else:
@@ -316,7 +290,8 @@ create an USB key or DVD image."))
 
         if self.bto_up:
             call = subprocess.Popen(['file', self.bto_up],
-                                    stdout=subprocess.PIPE)
+                                    stdout=subprocess.PIPE,
+                                    universal_newlines=True)
             output_text  = "<b>" + _("Utility Partition") + "</b>:\n"
             output_text += call.communicate()[0].replace(', ', '\n')
         else:
@@ -337,9 +312,9 @@ create an USB key or DVD image."))
         wizard.set_page_complete(base_page, False)
 
         if self.builder_widgets.get_object('iso_image_radio').get_active():
-            self.file_dialog.set_action(gtk.FILE_CHOOSER_ACTION_OPEN)
+            self.file_dialog.set_action(Gtk.FileChooserAction.OPEN)
         elif self.builder_widgets.get_object('directory_radio').get_active():
-            self.file_dialog.set_action(gtk.FILE_CHOOSER_ACTION_SELECT_FOLDER)
+            self.file_dialog.set_action(Gtk.FileChooserAction.SELECT_FOLDER)
         else:
             base_browse_button.set_sensitive(False)
             self.base_file_chooser_picked()
@@ -349,7 +324,6 @@ create an USB key or DVD image."))
 
         base_page = self.builder_widgets.get_object('base_page')
         wizard = self.widgets.get_object('wizard')
-
         wizard.set_page_complete(base_page, False)
 
         bto_version = ''
@@ -359,241 +333,55 @@ create an USB key or DVD image."))
         arch = ''
         if widget == self.builder_widgets.get_object('base_browse_button'):
             ret = self.run_file_dialog()
-            if ret is not None:
-                try:
-                    (bto_version, distributor, release, arch, output_text) = \
-                                       self.backend().query_iso_information(ret)
-
-                except dbus.DBusException, msg:
-                    parent = self.widgets.get_object('wizard')
-                    self.dbus_exception_handler(msg, parent)
-                self.bto_base = not not bto_version
-                self.builder_base_image = ret
-                wizard.set_page_complete(base_page, True)
         else:
+            ret = self.rp
+        
+        if ret is not None:
+            self.toggle_spinner_popup(True)
             try:
-                (bto_version, distributor, release, arch, output_text) = \
-                                   self.backend().query_iso_information(self.rp)
-
-                self.bto_base = not not bto_version
-                self.builder_base_image = self.rp
-                wizard.set_page_complete(base_page, True)
-            except dbus.DBusException, msg:
+                dbus_sync_call_signal_wrapper(self.backend(),
+                                            'query_iso_information',
+                                            {'report_iso_info': self.update_version_gui},
+                                            ret)
+            except dbus.DBusException as msg:
                 parent = self.widgets.get_object('wizard')
                 self.dbus_exception_handler(msg, parent)
-
-        if not bto_version:
-            bto_version = 'X00'
-
-        #set the version string that we fetched from the image
-        #or increment it if we started from a BTO base image
-        if self.bto_base:
-            bto_version = increment_bto_version(bto_version)
-        self.widgets.get_object('version').set_text(bto_version)
-
-        if distributor:
-            self.distributor = distributor
-        if release:
-            self.release = release
-        if arch:
-            self.arch = arch
-
-        #If this is a BTO image, then allow using built in framework
-        if output_text and \
-           not self.bto_base and \
-           self.builder_widgets.get_object('builtin_radio').get_active():
-            self.builder_widgets.get_object('git_radio').set_active(True)
-        self.builder_widgets.get_object('builtin_hbox').set_sensitive(self.bto_base)
-
-        self.builder_widgets.get_object('base_image_details_label').set_markup(output_text)
+            self.toggle_spinner_popup(False)
+            self.builder_base_image = ret
 
     def fid_toggled(self, widget):
         """Called when the radio button for the Builder FID overlay page is changed"""
         wizard = self.widgets.get_object('wizard')
         fid_page = self.builder_widgets.get_object('fid_page')
-        git_tree_hbox = self.builder_widgets.get_object('fid_git_tree_hbox')
         label = self.builder_widgets.get_object('fid_overlay_details_label')
 
-        label.set_markup("")
         wizard.set_page_complete(fid_page, False)
-        git_tree_hbox.set_sensitive(False)
+        self.builder_widgets.get_object('add_dell_recovery_button').set_sensitive(False)
 
         if self.builder_widgets.get_object('builtin_radio').get_active():
             wizard.set_page_complete(fid_page, True)
-            label.set_markup("<b>Builtin</b>: BTO Image")
-            self.builder_fid_overlay = ''
+            label.set_markup("<b>Builtin</b>: BTO Compatible Image")
+            self.add_dell_recovery_deb = ''
 
-        elif self.builder_widgets.get_object('git_radio').get_active():
-            git_tree_hbox.set_sensitive(True)
-            cwd = os.path.join(os.environ["HOME"],
-                             '.config',
-                             'dell-recovery',
-                             self.distributor + '-fid')
-            if os.path.exists(cwd) and os.path.exists('/usr/bin/git'):
-                self.fid_vte_handler(self.builder_widgets.get_object('git_radio'))
-
-    def fid_fetch_button_clicked(self, widget):
-        """Called when the button to test a git tree is clicked"""
-        wizard = self.widgets.get_object('wizard')
-        fid_page = self.builder_widgets.get_object('fid_page')
-        label = self.builder_widgets.get_object('fid_overlay_details_label')
-
-        if not os.path.exists('/usr/bin/git'):
-            output_text = _("<b>ERROR</b>: git is not installed")
-            if not self.apt_client:
-                try:
-                    self.apt_client = client.AptClient()
-                except NameError:
-                    pass
-            if self.apt_client:
-                self.builder_widgets.get_object('install_git_button').show()
-            wizard.set_page_complete(fid_page, False)
-        else:
-            output_text = ''
-            if not os.path.exists(os.path.join(os.environ['HOME'],
-                                               '.config',
-                                               'dell-recovery')):
-                os.makedirs(os.path.join(os.environ['HOME'],
-                                         '.config',
-                                         'dell-recovery'))
-            if not os.path.exists(os.path.join(os.environ['HOME'],
-                                               '.config',
-                                               'dell-recovery',
-                                               self.distributor + '-fid')):
-                command = ["git", "clone",
-                           self.builder_widgets.get_object('git_url').get_text(),
-                           os.path.join(os.environ["HOME"],
-                                        '.config',
-                                        'dell-recovery',
-                                      self.distributor + '-fid')]
-                cwd = os.path.join(os.environ["HOME"],
-                                   '.config',
-                                   'dell-recovery')
-            else:
-                command = ["git", "fetch", "--verbose"]
-                cwd = os.path.join(os.environ["HOME"],
-                                 '.config',
-                                 'dell-recovery',
-                                 self.distributor + '-fid')
-            self.widgets.get_object('wizard').set_sensitive(False)
-            self.builder_widgets.get_object('builder_vte_window').show()
-            self.git_pid = self.vte.fork_command(command = command[0],
-                                                 argv = command,
-                                                 directory = cwd)
-        label.set_markup(output_text)
-
-    def fid_fetch_cancel(self, widget):
-        """Handle a press to the cancel button of the VTE page"""
-        os.kill(self.git_pid, 15)
-
-    def fid_vte_handler(self, widget):
-        """Handler for VTE dialog closing"""
-        def fill_liststore_from_command(command, ffilter, liststore_name):
-            """Fills up the data in a liststore, only items matching filter"""
-            liststore = self.builder_widgets.get_object(liststore_name)
-            liststore.clear()
-            cwd = os.path.join(os.environ["HOME"],
-                               '.config',
-                               'dell-recovery',
-                               self.distributor + '-fid')
-            if not os.path.exists(cwd):
-                return
-            list_command = subprocess.Popen(args = command,
-                                            cwd = cwd,
-                                            stdout = subprocess.PIPE)
-            output = list_command.communicate()[0].split('\n')
-            #go through the list once to see if we have A rev tags at all
-            use_xrev = self.xrev
-            if not use_xrev:
-                use_xrev = True
-                for item in output:
-                    if ffilter + "_A" in item:
-                        use_xrev = False
-                        break
-            for item in output:
-                #Check that we have a valid item
-                # AND
-                #It doesn't contain HEAD
-                # AND
-                # [ We are in branch mode
-                #   OR
-                #   [
-                #     It contains our filter
-                #     We show X rev builds
-                #     It contains an X rev tag
-                #   ]
-                # ]
-
-                if item and \
-                   not "HEAD" in item and \
-                   (self.branch or \
-                   (ffilter in item and \
-                    (use_xrev or \
-                     not ffilter + "_X" in item))):
-                    liststore.append([item])
-
-            #Add this so that we can build w/o a tag only if we are in tag mode
-            # w/ dev on
-            if use_xrev and not self.branch:
-                liststore.append(['origin/master'])
-
-        #reactivate GUI
-        self.builder_widgets.get_object('builder_vte_window').hide()
-        self.widgets.get_object('wizard').set_sensitive(True)
-        self.builder_widgets.get_object('fid_git_tag_hbox').set_sensitive(True)
-
-        #update the tag list in the GUI
-        if self.branch:
-            command = ["git", "branch", "-r"]
-        else:
-            command = ["git", "tag", "-l"]
-        fill_liststore_from_command(command, self.release, 'tag_liststore')
-
-    def fid_git_changed(self, widget):
-        """If we have selected a tag"""
-        wizard = self.widgets.get_object('wizard')
-        fid_page = self.builder_widgets.get_object('fid_page')
-
-        active_iter = self.builder_widgets.get_object('git_tags').get_active_iter()
-        active_tag = ''
-        output_text = ''
-        if active_iter:
-            active_tag = self.builder_widgets.get_object('tag_liststore').get_value(
-                active_iter, 0)
-
-        if active_tag:
-            cwd = os.path.join(os.environ["HOME"],
-                               '.config',
-                               'dell-recovery',
-                               self.distributor + '-fid')
-            #switch checkout branches
-            command = ["git", "checkout", active_tag.strip()]
-            subprocess.call(command, cwd = cwd)
-
-            self.builder_fid_overlay = os.path.join(cwd, 'framework')
-
-            tag = active_tag.strip().split('_')
-            if len(tag) > 1:
-                self.widgets.get_object('version').set_text(tag[1])
-            else:
-                self.widgets.get_object('version').set_text('X00')
-
-            output_text = "<b>GIT Tree</b>, Version: %s" % active_tag
-
-            #if we have a valid tag, check now to make sure that we have
-            # the dell-recovery deb
-            if self.backend().query_have_dell_recovery(self.builder_base_image,
-                        self.builder_fid_overlay) or self.add_dell_recovery_deb:
+        elif self.builder_widgets.get_object('deb_radio').get_active():
+            if self.add_dell_recovery_deb:
                 wizard.set_page_complete(fid_page, True)
             else:
-                output_text += "\n<b>%s</b>, %s" % (_("Missing Dell-Recovery"),
-                               _("Not present in ISO or GIT tree"))
-                wizard.set_page_complete(fid_page, False)
-                self.builder_widgets.get_object('add_dell_recovery_button').show()
-            
-        else:
-            wizard.set_page_complete(fid_page, False)
-        self.builder_widgets.get_object('fid_overlay_details_label').set_markup(output_text)
+                label.set_markup("")
+            self.builder_widgets.get_object('add_dell_recovery_button').set_sensitive(True)
+
+    def fid_deb_changed(self, widget):
+        """Detects the version of a newly found deb"""
+        wizard = self.widgets.get_object('wizard')
+        fid_page = self.builder_widgets.get_object('fid_page')
+        wizard.set_page_complete(fid_page, False)
+        if self.add_dell_recovery_deb:
+            wizard.set_page_complete(fid_page, True)
+            if self.add_dell_recovery_deb == 'dpkg-repack':
+                output_text = "<b>Build</b> from on-system install"
+            else:
+                output_text = "<b>Add</b> from %s" % self.add_dell_recovery_deb
+            self.builder_widgets.get_object('fid_overlay_details_label').set_markup(output_text)
 
     def driver_action(self, widget):
         """Called when the add or remove buttons are pressed on the driver action page"""
@@ -708,13 +496,11 @@ create an USB key or DVD image."))
         return proceed
 
     def install_app(self, widget):
-        """Launch into an installer for git or dpkg-repack"""
+        """Launch into an installer for dpkg-repack"""
         packages = []
         wizard = self.widgets.get_object('wizard')
-        if widget == self.builder_widgets.get_object('install_git_button'):
-            packages = ['git-core']
-        else:
-            packages = ['dpkg-repack']
+        
+        packages = ['dpkg-repack']
         try:
             trans = self.apt_client.install_packages(packages,
                                     wait=False,
@@ -724,14 +510,13 @@ create an USB key or DVD image."))
             dialog = AptProgressDialog(trans, parent=wizard)
             dialog.run()
             super(AptProgressDialog, dialog).run()
-        except dbus.exceptions.DBusException, msg:
+        except dbus.exceptions.DBusException as msg:
             self.dbus_exception_handler(msg, wizard)
 
         widget.hide()
 
     def add_dell_recovery_clicked(self, widget):
         """Callback to launch a dialog to add dell-recovery to the image"""
-        widget.hide()
         #check if dpkg-repack is available
         if not os.path.exists('/usr/bin/dpkg-repack'):
             if not self.apt_client:
@@ -759,7 +544,10 @@ create an USB key or DVD image."))
             wizard.set_page_complete(fid_page, False)
             self.add_dell_recovery_deb = ''
         window.hide()
-        self.fid_git_changed(None)
+
+        #validate version
+        if self.builder_widgets.get_object('deb_radio').get_active():
+            self.fid_deb_changed(None)
 
     def add_dell_recovery_toggled(self, widget):
         """Toggles the active selection in the add dell-recovery to image page"""
@@ -781,7 +569,7 @@ create an USB key or DVD image."))
         """Called when a file is selected on the add dell-recovery page"""
 
         ok_button = self.builder_widgets.get_object('builder_add_ok')
-        filefilter = gtk.FileFilter()
+        filefilter = Gtk.FileFilter()
         filefilter.add_pattern("*.deb")
         self.file_dialog.set_filter(filefilter)
             
@@ -789,8 +577,8 @@ create an USB key or DVD image."))
         if ret is not None:
             import apt_inst
             import apt_pkg
-            control = apt_inst.debExtractControl(open(ret))
-            sections = apt_pkg.ParseSection(control)
+            control = apt_inst.DebFile(ret).control.extractdata("control")
+            sections = apt_pkg.TagSection(control)
             if sections["Package"] != 'dell-recovery':
                 self.add_dell_recovery_deb = ''
             else:
@@ -798,3 +586,19 @@ create an USB key or DVD image."))
 
         if self.add_dell_recovery_deb:
             ok_button.set_sensitive(True)
+
+    def update_version_gui(self, version, distributor, release, arch, output_text):
+        """Stops any running spinners and updates GUI items"""
+        BasicGeneratorGTK.update_version_gui(self, version, distributor, release, arch, output_text)
+
+        if output_text:
+            complete = (output_text != "Unknown Base Image")
+            base_page = self.builder_widgets.get_object('base_page')
+            wizard = self.widgets.get_object('wizard')
+            wizard.set_page_complete(base_page, complete)
+            #If this is a BTO image, then allow using built in framework
+            if not self.bto_base and self.builder_widgets.get_object('builtin_radio').get_active():
+                self.builder_widgets.get_object('deb_radio').set_active(True)
+        self.builder_widgets.get_object('builtin_hbox').set_sensitive(self.bto_base)
+
+        self.builder_widgets.get_object('base_image_details_label').set_markup(output_text)
