@@ -1,4 +1,4 @@
-#!/usr/bin/python
+#!/usr/bin/python3
 # -*- coding: utf-8 -*-
 #
 # «recovery_gtk» - Dell Recovery GTK Frontend
@@ -27,11 +27,10 @@ import os
 import subprocess
 import dbus
 import sys
-import glib
 
-import gtk
+from gi.repository import Gtk,GLib
 
-from Dell.recovery_common import (DOMAIN, LOCALEDIR, UIDIR, DBUS_INTERFACE_NAME,
+from Dell.recovery_common import (DOMAIN, LOCALEDIR, UIDIR, SVGDIR, DBUS_INTERFACE_NAME,
                                   DBUS_BUS_NAME, dbus_sync_call_signal_wrapper,
                                   PermissionDeniedByPolicy, check_version)
 
@@ -41,22 +40,33 @@ from gettext import bindtextdomain, textdomain
 
 class DellRecoveryToolGTK:
     """GTK implementation of the Dell Recovery suite for Linux"""
-    def __init__(self, recovery):
+    def __init__(self, recovery, mode='recovery'):
+        def action_objects(widgets, objects, action):
+            for item in ['button', 'image', 'label']:
+                if action == 'hide':
+                    widgets.get_object(objects + '_' + item).hide()
+                else:
+                    widgets.get_object(objects + '_' + item).show_all()
 
         #setup locales
         bindtextdomain(DOMAIN, LOCALEDIR)
         textdomain(DOMAIN)
-        self.tool_widgets = gtk.Builder()
+        self.tool_widgets = Gtk.Builder()
         self.tool_widgets.add_from_file(os.path.join(UIDIR, 'tool_selector.ui'))
-        gtk.window_set_default_icon_from_file('/usr/share/pixmaps/dell-dvd.svg')
+        Gtk.Window.set_default_icon_from_file(os.path.join(SVGDIR, 'dell-dvd.svg'))
 
         translate_widgets(self.tool_widgets)
         self.tool_widgets.connect_signals(self)
 
-        #hide restore from HDD unless there is a recovery partition
-        if not (recovery and os.path.exists('/etc/grub.d/99_dell_recovery')):
-            for item in ['button', 'image', 'label']:
-                self.tool_widgets.get_object('restore_system_' + item).hide()
+        #if running in driver install mode,  hide other stuff
+        if mode == 'driver':
+            for item in ['restore_system', 'build_os_media']:
+                action_objects(self.tool_widgets, item, 'hide')
+                action_objects(self.tool_widgets, 'install_drivers', 'show')
+        else:
+            #hide restore from HDD unless there is a recovery partition
+            if not (recovery and os.path.exists('/etc/grub.d/99_dell_recovery')):
+                action_objects(self.tool_widgets, 'restore_system', 'hide')
 
         #about dialog
         self.about_box = None
@@ -77,11 +87,11 @@ class DellRecoveryToolGTK:
                 self._dbus_iface = dbus.Interface(bus.get_object(DBUS_BUS_NAME,
                                                   '/RecoveryMedia'),
                                                   DBUS_INTERFACE_NAME)
-            except dbus.DBusException, msg:
+            except dbus.DBusException as msg:
                 self.dbus_exception_handler(msg)
                 sys.exit(1)
-            except Exception, msg:
-                self.show_alert(gtk.MESSAGE_ERROR, "Exception", str(msg),
+            except Exception as msg:
+                self.show_alert(Gtk.MessageType.ERROR, "Exception", str(msg),
                            parent=self.tool_widgets.get_object('tool_selector'))
 
         return self._dbus_iface
@@ -98,7 +108,7 @@ class DellRecoveryToolGTK:
         if not parent:
             parent = self.tool_widgets.get_object('tool_selector')
 
-        self.show_alert(gtk.MESSAGE_ERROR, _("Exception"), text, parent)
+        self.show_alert(Gtk.MessageType.ERROR, _("Exception"), text, parent)
 
         if fallback:
             parent.hide()
@@ -121,14 +131,15 @@ class DellRecoveryToolGTK:
                 iface = dbus.Interface(obj, 'org.gnome.SessionManager')
                 iface.RequestReboot()
                 self.destroy()
-            except dbus.DBusException, msg:
+            except dbus.DBusException as msg:
                 self.dbus_exception_handler(msg)
             tool_selector.set_sensitive(True)
             
             #don't do further processing
             return False
-        #Restore Media Button
-        elif widget == self.tool_widgets.get_object('build_os_media_button'):
+        #Restore Media Button / Driver install button
+        elif widget == self.tool_widgets.get_object('build_os_media_button') or \
+             widget == self.tool_widgets.get_object('install_drivers_button'):
             self.tool_widgets.get_object('tool_selector').set_sensitive(False)
 
             #continue
@@ -140,14 +151,14 @@ class DellRecoveryToolGTK:
             # run yelp
             proc = subprocess.Popen(["yelp", "ghelp:dell-recovery"])
             # collect the exit status (otherwise we leave zombies)
-            glib.timeout_add_seconds(1, lambda proc: proc.poll() == None, proc)
+            GLib.timeout_add_seconds(1, lambda proc: proc.poll() == None, proc)
         elif widget == self.tool_widgets.get_object('about_menu_item'):
             tool_selector = self.tool_widgets.get_object('tool_selector')
             if not self.about_box:
-                self.about_box = gtk.AboutDialog()
+                self.about_box = Gtk.AboutDialog()
                 self.about_box.set_version(check_version())
                 self.about_box.set_name(_("Dell Recovery"))
-                self.about_box.set_copyright(_("Copyright 2008-2010 Dell Inc."))
+                self.about_box.set_copyright(_("Copyright 2008-2012 Dell Inc."))
                 self.about_box.set_website("http://www.dell.com/ubuntu")
                 self.about_box.set_authors(["Mario Limonciello"])
                 self.about_box.set_destroy_with_parent(True)
@@ -166,7 +177,7 @@ class DellRecoveryToolGTK:
     def run(self):
         """Runs the GTK application's main functions"""
         self.tool_widgets.get_object('tool_selector').show()
-        gtk.main()
+        Gtk.main()
 
     def show_alert(self, alert_type, header, body=None, parent=None):
         """Displays an alert message"""
@@ -183,47 +194,52 @@ class DellRecoveryToolGTK:
         message = "<b><big>%s</big></b>" % header
         if not body == None:
             message = "%s\n\n%s" % (message, body)
+            print(body, file=sys.stderr)
         label_hig.set_markup(message)
         
-        if alert_type == gtk.MESSAGE_ERROR:
+        if alert_type == Gtk.MessageType.ERROR:
             image_hig.set_property("stock", "gtk-dialog-error")
-        elif alert_type == gtk.MESSAGE_WARNING:
+        elif alert_type == Gtk.MessageType.WARNING:
             image_hig.set_property("stock", "gtk-dialog-warning")
-        elif alert_type == gtk.MESSAGE_INFO:
+        elif alert_type == Gtk.MessageType.INFO:
             image_hig.set_property("stock", "gtk-dialog-info")
 
         res = self.tool_widgets.get_object('dialog_hig').run()
         self.tool_widgets.get_object('dialog_hig').hide()
-        if res == gtk.RESPONSE_CLOSE:
+        if res == Gtk.ResponseType.CLOSE:
             return True
         return False
 
-    def destroy(self, widget=None, data=None):
-        """Closes any open backend connections and stops GTK threads"""
+    def cleanup_backend(self, widget=None, data=None):
+        '''cleans up the backend process'''
         try:
             if self._dbus_iface is not None:
                 self.backend().request_exit()
-        except dbus.DBusException, msg:
+        except dbus.DBusException as msg:
             if hasattr(msg, '_dbus_error_name') and msg.get_dbus_name() == \
                     'org.freedesktop.DBus.Error.ServiceUnknown':
                 pass
             else:
-                print "%s when closing DBus service from %s (data: %s)" % \
-                                             (str(msg), widget.get_name(), data)
-        gtk.main_quit()
+                print("%s when closing DBus service from %s (data: %s)" %
+                      (str(msg), widget, data))
+
+    def destroy(self, widget=None, data=None):
+        """Closes any open backend connections and stops GTK threads"""
+        self.cleanup_backend(widget, data)
+        Gtk.main_quit()
 
 def translate_widgets(widgets):
     """Translates all widgets to the specified domain"""
     widgets.set_translation_domain(DOMAIN)
     for widget in widgets.get_objects():
-        if isinstance(widget, gtk.Label):
+        if isinstance(widget, Gtk.Label):
             widget.set_property('can-focus', False)
             widget.set_text(_(widget.get_text()))
-        elif isinstance(widget, gtk.RadioButton):
+        elif isinstance(widget, Gtk.RadioButton):
             widget.set_label(_(widget.get_label()))
-        elif isinstance(widget, gtk.Button):
+        elif isinstance(widget, Gtk.Button):
             widget.set_label(_(widget.get_label()))
-        elif isinstance(widget, gtk.Window):
+        elif isinstance(widget, Gtk.Window):
             title = widget.get_title()
             if title:
                 widget.set_title(_(widget.get_title()))
